; Obsluha cidla DS1820
; Vypracoval Vladimir Visnovsky

; Hlavne nastavenia
; - Port
.Equ	dsPort = PortD
; - Pin
.Equ	dsPin  = 0
; Vystup v XH/XL

; Procedury
; !!! nutna kniznica wMega - (pouzita funkcia Delay_us)
; DS_Init    -> nasatvy na 9 bit konverziu
; DS_Reset   -> restart
; DS_Load    -> V HL vracia hodnotu 
; DS_Load_1s -> Najprv precita udaje a potom da povel na konverziu
;               - Tento prikaz sa vykonava kazdu sekundu.  
; Adresa DS_Error, ak cidlo neodpoveda, je volana RCallom
; Flag Register
; F : 0 - 1/2
; F : 1 - Minus
; F : 2 - Architektura ..B..
; F : 3 - Cidlo nieje pritomne

; Preskoc procedury
Rjmp	ds_End

; Makra 
.Macro DS_L
 Sbi	dsPort-1	, dsPin
.EndMacro
.Macro DS_H
 Cbi	dsPort-1	, dsPin
.EndMacro

; RESET
dsReset:
 Cli
 ; Stiahnutie zbernice na 480us
 DS_L					
 mDelay_us [600]
 ; Povolenie zbernice na H
 DS_H
 mDelay_us [80]
 ; Odtestovanie prezenatcie zbernice
 In	  T		, dsPort-2
 Andi  F		, 0b11110111
  Sbrc T		, dsPin
   Ori F		, 0b00001000
 Sei
 mDelay_us [600]
 Ret;
; V  podobe makra
.Macro DS_Reset
 RCall dsReset
.EndMacro 

; Nacitanie Byte
DS_Read_Byte:			 ; (A : Byte)
  Ldi 	B		, 9		 ; 8 Bitov
  DS_Read_Byte_Loop:	 ; Slucka
   Dec	B				 ; Odpocitanie jednej
   Breq DS_Read_Byte_End ; Ak prejde nulov tak ukonc proceduru 
   ; Nacitanie bitu
   Lsr	A				 ; Posuv A8 -> A7...
   ;DS_ReadBite   
    Cli
    ; Stiahnutie zbernice na 5us
    DS_L					
    mDelay_us [5] ; 5
    ; Povolenie zbernice na H
    DS_H
    mDelay_us [10]
    ; Nacitanie premennej
    In	T		, dsPort-2
    Sei
    mDelay_us [60]
   ; Dekodovanie 	premmenej 
   Andi	A		, 0b01111111 ; Vynulovanie 7 bitu
   Sbrc T		, dsPin
    Ori	A		, 0b10000000 ; Nastavenie 7 bitu
   ; - Koniec DS_ReadBite
   Rjmp	DS_Read_Byte_Loop
  DS_Read_Byte_End:  
 Ret
 ; V podobe makra
 .Macro DS_Read
  RCall DS_Read_Byte
 .EndMacro 
 
; Ulozenie Byte
DS_Write_Byte:			 ; (A : Byte)
  Ldi 	B		, 9		 ; 8 Bitov
  DS_Write_Byte_Loop:	 ; Slucka
   Dec	B				 ; Odpocitanie jednej
   Breq DS_Write_Byte_End ; Ak prejde nulov tak ukonc proceduru 
   ; Nacitanie bitu
   ;DS_SendBite
    Cli
    Sbrc	A		, 0
     Rjmp DS_SendBite_H
     ; Zapis Log. 0
     ; Stiahnutie zbernice na 55us
     DS_L					
     mDelay_us [45]
     ; Povolenie zbernice na H
     DS_H
     mDelay_us [20] ; Bolo 5
     ; koniec
    Rjmp DS_SendBite_End
     ; Zapis Log. 1
     DS_SendBite_H:
     ; Stiahnutie zbernice na 10us
     DS_L					
     mDelay_us [10] ; bolo 10
     ; Povolenie zbernice na H
     DS_H
     mDelay_us [55]
     ; Koniec
    DS_SendBite_End:
   Sei
   ; - End DS_SendBite
   Lsr	A				 ; Posuv A2 -> A1...
   Rjmp	DS_Write_Byte_Loop   
  DS_Write_Byte_End:  
 Ret
 ; V podobe makra
 .Macro DS_Write
  RCall DS_Write_Byte
 .EndMacro 
 .Macro DS_Write_i
  Ldi A		, @0
  RCall DS_Write_Byte
 .EndMacro 

; Inicializacia DS
dsInit:
 ; Nastavenie prostredia
  Cbi	dsPort	, dsPin
 ; Vynulovanie Typu z Flag
  Andi  F		, 0b11111011
 ; Nacitanie typu
  DS_Reset			    ; Restart
  DS_Write [0x33]		; Citanie z ROM
  Delay_us [10]
  DS_Read				; Nacitanie typu 
  Sbrs A		, 4 	; Ak je 10h preskoc Lebo 28H nema zdvihnuty ten bit	
   Ori F		, 0b00000100  
 ; Nulovanie registrov 
 DS_Reset				; Restart
 DS_Write [0xCC]		; Skip ROM
 DS_Write [0x4E]		; Zapis do tabulky EEPROM
 DS_Write [0x00]		; TH
 DS_Write [0x00]		; TL
  Sbrs F	, 2			; Plati pre typ B
   Rjmp dsInit_S
    DS_Write [0x1F]		; 9. bit [0b00011111]
  dsInit_S:
  ; Koniec
  DS_Reset
 Ret
 ; Makro Init 
.Macro DS_Init
 RCall dsInit
.EndMacro 

; Procedura konvertovania
dsRead:
 ; Nastavenie prostredia
 ; Program
 ;Cli
  ; Konvertovanie 
  Rcall DSReset			; Restart
  Acc [0xCC]			; Skip ROM
  Rcall DS_Write_Byte	; Odosli
  Acc [0x44]			; Convert
  Rcall DS_Write_Byte	; Odosli
  ; Dlaha pauza dovolime prerusenie
 ;Pop B
 ;Sei 
  ; Pauza 100ms
  Ldi 	B		, 255   ; 25 x 4ms -> >100ms
  DS_Read_Loop:	 		; Slucka
   Dec	B		 		; Odpocitanie jednej
   Breq DS_Read_End 	; Ak prejde nulov tak ukonc proceduru 
   ; Cakaj 4us
    Delay_us [4000]
   Rjmp	DS_Read_Loop
  DS_Read_End:
  ; Koniec dlhej pauzi, zakazanie prerusenia  
 ;Cli
 ;Push B
  ; Nacitanie A
  Rcall DSReset			; Restart
  Acc [0xCC]			; Skip ROM
  Rcall DS_Write_Byte	; Odosli
  Acc [0xBE]			; Nacitanie hodnot
  Rcall DS_Write_Byte	; Odosli
  Rcall DS_Read_Byte	; Nacitanie spodneho bytu
  Mov XL		, A
  Delay_us [10]
  Rcall DS_Read_Byte	; Nacitanie horneho bytu
  Mov XH		, A
  ; Koniec
  Rcall DSReset		; Restart  
  ;Sei
 ; Ukoncenie prostredia
 ;Pop B
 ;PopA
 Ret
.Macro DS_Load
 RCall dsRead
.EndMacro 

; Procedura konvertovania
dsRead_1S:
  ; Nacitanie A
  Rcall DSReset			; Restart
  Acc [0xCC]			; Skip ROM
  Rcall DS_Write_Byte	; Odosli
  Acc [0xBE]			; Nacitanie hodnot
  Rcall DS_Write_Byte	; Odosli
  Rcall DS_Read_Byte	; Nacitanie spodneho bytu
  Mov XL		, A
  Delay_us [10]
  Rcall DS_Read_Byte	; Nacitanie horneho bytu
  Mov XH		, A
  ; Konvertovanie 
  Rcall DSReset			; Restart
  Acc [0xCC]			; Skip ROM
  Rcall DS_Write_Byte	; Odosli
  Acc [0x44]			; Convert
  Rcall DS_Write_Byte	; Odosli
  ; Koniec
  Rcall DSReset		; Restart  
 ; Ukoncenie prostredia
 Ret
.Macro DS_Load_1S
 RCall dsRead_1S
.EndMacro 

;Makro konverzie
dsDS_Get:
 Sbrc F		, 2
  Rjmp dsDS_Get_B
 ; Typ S
 ; Vlozenie Vlajok
 Bst  XH		, 7					; Priznak zapornej hodnoty 
 Bld  F			, 1					; - vloz pod vlajkov
 Bst  XL		, 0					; Hodnotu pol stupna 
 Bld  F			, 0					; - vloz pod vlajkov
 ; Zaporna hodnota
  Acc [0]							; Nula
 Sbrc F			, 1					; Zisti ci je zaporna hodnota
  Sub A			, XL				; - Ak ano, prepocitaj
 Sbrs F			, 1					; Zisti ci je zaporna hodnota
  Mov A			, XL
 ; Vypustenie pol stupna
 Lsr  A								; Vypustenie pol stupna 				  
 Ret
 ; Typ B
 dsDS_Get_B:
 ; Vlozenie Vlajok
 Bst  XH		, 7					; Priznak zapornej hodnoty 
 Bld  F			, 1					; - vloz pod vlajkov
 Bst  XL		, 3					; Hodnotu pol stupna 
 Bld  F			, 0					; - vloz pod vlajkov
 ; Posuv na typ S
 Ror XH
 Ror XL
 Ror XH
 Ror XL
 Ror XH
 Ror XL
 ; Zaporna hodnota
  Acc [0]							; Nula
 Sbrc F			, 1					; Zisti ci je zaporna hodnota
  Sub A			, XL				; - Ak ano, prepocitaj
 Sbrs F			, 1					; Zisti ci je zaporna hodnota
  Mov A			, XL
 ; Vypustenie pol stupna
 Lsr  A								; Vypustenie pol stupna 				  
 Ret
.Macro DS_Get 
 RCall dsDS_Get
.EndMacro

; Koniec kniznice
ds_End:
